#python DEEP.py <Test data>

# gensim modules
from gensim import utils
from gensim.models.doc2vec import TaggedDocument,LabeledSentence
from gensim.models import Doc2Vec

# numpy
import numpy as np

# classifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
# utils
import random
from os import listdir
import sys


def get_prot_label_result(X,classifier,filename):
   
    fout = open(filename,"w")
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
    #probs = classifier.predict_proba(X)
    for i in range(len(X)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        text = org_feature_to_text[x_str]
        #text = bow_to_text[bow_str]
        prots = org_text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 
    
    for prot in prot_predicted_label:
        
        MP_count = sum(prot_predicted_label[prot])
#        if MP_count >= len(prot_predicted_label[prot])-MP_count:	#for 50% majority vote	
        if MP_count > len(prot_predicted_label[prot]) * 0.8:
            fout.write(prot+" 1\n")
        else:
            fout.write(prot+" 0\n")

class LabeledLineSentence(object):
    def __init__(self, sources):
        self.sources = sources

        flipped = {}

        # make sure that keys are unique
        for key, value in sources.items():
            if value not in flipped:
                flipped[value] = [key]
            else:
                raise Exception('Non-unique prefix encountered')

    def __iter__(self):
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    yield TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no])

    def to_array(self):
        self.sentences = []
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    self.sentences.append(TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no]))
        return self.sentences

    def sentences_perm(self):
        random.shuffle(self.sentences)
        return self.sentences


sources = {}
for f in listdir("Deep_data"):
    #print f
    if not sources.has_key(f):
        sources["Deep_data/"+f]=f

sentences = LabeledLineSentence(sources)

model = Doc2Vec(min_count = 4, window = 4, size = 100, dm = 2, sample = 1e-3, negative = 0, workers = 7)
model.build_vocab(sentences.to_array())

for epoch in range(1000):
    model.train(sentences.sentences_perm())

print ("Done Model Training")

Xtest = []
organism_data = open(sys.argv[1],"r")
org_text_to_prot = {}
org_feature_to_text = {}
org_texts = []
while 1:
    line = organism_data.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not org_text_to_prot.has_key(text):
        org_text_to_prot[text] = set()
        org_text_to_prot[text].add(id)
        
        l = utils.to_unicode(text).split()
        x =  model.infer_vector(l)
        Xtest.append(x)
        
        x_str = " ".join([str(j) for j in x])
        org_feature_to_text[x_str]=text
        #texts.append(text.split())
    else:
        org_text_to_prot[text].add(id)
    
        
X = []
y = []
MP_file = open("MP_Desc.dat","r")
text_to_prot = {}
text_to_label = {}
feature_to_text = {}
texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        l = utils.to_unicode(text).split()
        x =  model.infer_vector(l)
        X.append(x)
        y.append(1)
        x_str = " ".join([str(j) for j in x])
        feature_to_text[x_str]=text
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open("MP_Desc.dat","r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        l = utils.to_unicode(text).split()
        x =  model.infer_vector(l)
        X.append(x)
        y.append(0)
        x_str = " ".join([str(j) for j in x])
        feature_to_text[x_str]=text
    else:
        text_to_prot[text].add(id)

print ("Done feature generation using deep model")
classifier = LogisticRegression(C = 0.5, solver = 'liblinear', max_iter = 100)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'DEEP_logistic_'+sys.argv[1])

classifier =RandomForestClassifier(n_estimators = 750)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'DEEP_randomforest_'+sys.argv[1])  

classifier = GradientBoostingClassifier(learning_rate=0.2, n_estimators = 250)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'DEEP_gradientboost_'+sys.argv[1]) 

classifier = SVC(kernel = 'rbf', C = 1, gamma = 0.1)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'DEEP_SVM_'+sys.argv[1]) 

print ("Done prediction on new test data")

