#python LDA.py <Test file>

import logging
import sys
from gensim import corpora, models, similarities
from pprint import pprint
from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
import numpy as np

logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)

def get_label_text(bow_str):

    text = bow_to_text[bow_str]
    label = text_to_label[text]
    return label

def get_prot_label_result(X,classifier,filename):
   
    fout = open(filename,"w")
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
    #probs = classifier.predict_proba(X)
    for i in range(len(X)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = org_feature_to_bow[x_str]
        text = org_bow_to_text[bow_str]
        prots = org_text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 
    
    for prot in prot_predicted_label:
        
        MP_count = sum(prot_predicted_label[prot])
        if MP_count >= len(prot_predicted_label[prot])-MP_count:
            fout.write(prot+" 1\n")
        else:
            fout.write(prot+" 0\n")            

organism_data = open(sys.argv[1],"r")
org_text_to_prot = {}
org_texts = []
while 1:
    line = organism_data.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not org_text_to_prot.has_key(text):
        org_text_to_prot[text] = set()
        org_text_to_prot[text].add(id)
        org_texts.append(text.split())
    else:
        org_text_to_prot[text].add(id)   

MP_file = open("MP_Desc.dat","r")
text_to_prot = {}
text_to_label = {}
texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open("NMP_Desc.dat","r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

print ("Done reading training data")

dictionary = corpora.Dictionary(texts)
bow_to_text = {}

corpus = [] 
for text in texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not bow_to_text.has_key(store_text):
        bow_to_text[str(bow)] = store_text
    corpus.append(bow)

org_bow_to_text = {}

org_corpus = [] 
for text in org_texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not org_bow_to_text.has_key(store_text):
        org_bow_to_text[str(bow)] = store_text
    org_corpus.append(bow)

tfidf = models.TfidfModel(corpus)
corpus_tfidf = tfidf[corpus]
num_topics = 90
topic = models.LdaModel(corpus_tfidf,id2word=dictionary,num_topics=num_topics,iterations=5000)

print ("Done Populating LDA Model")

X = []
y = []
feature_to_bow = {}
for bow in corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    topic_dist = dict(topic.get_document_topics(doc,minimum_probability=0.000))
    label = get_label_text(bow_str)
    #print bow_to_text[bow_str],get_label_text(bow_str)
    y.append(label)
    x = []
    for id in range(0,num_topics):
        #print word,id
        if topic_dist.has_key(id):
            x.append(topic_dist[id])
        else:
            #output.write("0 ")
            x.append(0)
    x_str = " ".join([str(i) for i in x])
    feature_to_bow[x_str]=bow_str
    X.append(x)

#exit()
print np.asarray(X).shape
print np.sum(y)

Xtest = []
org_feature_to_bow = {}
for bow in org_corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    topic_dist = dict(topic.get_document_topics(doc,minimum_probability=0.000))
    x = []
    for id in range(0,num_topics):
        #print word,id
        if topic_dist.has_key(id):
            x.append(topic_dist[id])
        else:
            #output.write("0 ")
            x.append(0)
    x_str = " ".join([str(i) for i in x])
    org_feature_to_bow[x_str]=bow_str
    Xtest.append(x)

print ("Done generating LDA-based feature vectors")


classifier = LogisticRegression(C = 1, solver = 'liblinear', max_iter = 100)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'LDA_logistic_'+sys.argv[1])

classifier = RandomForestClassifier(n_estimators = 750)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'LDA_randomforest_'+sys.argv[1])  

classifier =  GradientBoostingClassifier(learning_rate=0.2, n_estimators = 500)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'LDA_gradientboost_'+sys.argv[1]) 

classifier = SVC(kernel = 'rbf', C = 5, gamma = 0.1)	
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'LDA_SVM_'+sys.argv[1]) 

print ("Done predictiong for new data")

