# USAGE:
# python LDA.py <MP File> <Non_MP FILE>

#example:

# python LDA_GBM.py ../../Data/Cleaned/Title/title_MP.dat.processed ../../Data/Cleaned/Title/title_NMP.dat.processed
# Similar for abstract and Functional Description


import sys
from gensim import corpora, models, similarities
from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
import numpy as np
import time


MP_file = open(sys.argv[1],"r")
protein_to_cls = {}
protein_id_to_cls = {}
texts = []
model_time = 0.0
infer_time = 0.0
CVtime = []
index = 1
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    protein_to_cls[id]=index
    protein_id_to_cls[index]=1
    index+=1
    texts.append(text.split())
    
NonMP_file = open(sys.argv[2],"r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    protein_to_cls[id]=index
    protein_id_to_cls[index]=0
    index+=1
    texts.append(text.split())


start = time.time()
dictionary = corpora.Dictionary(texts)
corpus = [] 
for text in texts:
    corpus.append(dictionary.doc2bow(text))

tfidf = models.TfidfModel(corpus)
corpus_tfidf = tfidf[corpus]



if "title" in sys.argv[1]:
    num_topics = 110     # title
elif "abstract" in sys.argv[1]:
    num_topics = 110	#abstract
else:
    num_topics = 110     #function


topic = models.LdaModel(corpus_tfidf,id2word=dictionary,num_topics=num_topics,iterations=5000)
end = time.time()
model_time = (float(end - start))
print "Num_Topics = " + str(num_topics);
index=1
X = []
y = []
start = time.time()
for doc in corpus_tfidf:
    y.append(protein_id_to_cls[index])
    x = []
    topic_dist = dict(topic.get_document_topics(doc,minimum_probability=0.000))
    for id in range(0,num_topics):
	
	if topic_dist.has_key(id):
	    x.append(topic_dist[id])
	else:	    
	    x.append(0)
    X.append(x)
    index+=1
	   
end = time.time()
infer_time = (float(end - start))
print model_time, infer_time
        

f1GBM = []
preGBM = []
recalGBM = []

for c in range(5):
    print "fold = ", c

    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

	    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
	continue
    if np.sum(y_test) == 0:		#only 0 class
	continue

    T = len(y_test)
    P = sum(y_test)
    N = T - P
    start = time.time()


    if "title" in sys.argv[1]:
        cls = GradientBoostingClassifier(learning_rate=0.3, n_estimators = 100)	#title
    elif "abstract" in sys.argv[1]:
        cls = GradientBoostingClassifier(learning_rate=0.2, n_estimators = 500)	#abstract
    else:
        cls = GradientBoostingClassifier(learning_rate=0.3, n_estimators = 100)

    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)
    preGBM.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalGBM.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1GBM.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done GBM"

    end = time.time()
    CVtime.append(float(end - start))
    fout.write("Num_Topics = " + str(num_topics) + "\n");

fout=open('Result_LDA_function_GBM.txt', 'w')


fout.write("\n5 fold GBM:");
fout.write("F1 = %f\n" % np.mean(f1GBM))
fout.write("precision = %f\n" % np.mean(preGBM))
fout.write("recall = %f\n" % np.mean(recalGBM))

time =  model_time + infer_time +np.mean(CVtime)
fout.write("Model time = "+ str(model_time) +" Sec, Infer time = "+ str(infer_time) +" Sec, Classification time =" + str(np.mean(CVtime))+ "Sec\n")
fout.write("Total time = %f Sec\n" % time)
fout.close();

