#USAGE:
# python para2vec_deep_learning.py <input dir>
# input dir should contain two files named 0 and 1
# 0 named file contain Non Moonlighting protein text data
# 1 named file contain Moonlighting protein text data
 
#example:

#python para2vec_deep_learning_SVM.py inputdir_for_deep

# gensim modules version 0.12.1
from gensim import utils
from gensim.models.doc2vec import TaggedDocument,LabeledSentence
from gensim.models import Doc2Vec

# numpy
import numpy as np

# classifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn import metrics
from sklearn.cluster import KMeans
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.svm import SVC
# utils
import random
from os import listdir
import sys
import time



class LabeledLineSentence(object):
    def __init__(self, sources):
        self.sources = sources

        flipped = {}

        # make sure that keys are unique
        for key, value in sources.items():
            if value not in flipped:
                flipped[value] = [key]
            else:
                raise Exception('Non-unique prefix encountered')

    def __iter__(self):
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    line = line.strip().split("\t")[1]
                    yield TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no])

    def to_array(self):
        self.sentences = []
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    line = line.strip().split("\t")[1]
                    self.sentences.append(TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no]))
        return self.sentences

    def sentences_perm(self):
        random.shuffle(self.sentences)
        return self.sentences

sources = {}
for f in listdir(sys.argv[1]):
    #print f
    if not sources.has_key(f):
        sources[sys.argv[1]+"/"+f]=f

sentences = LabeledLineSentence(sources)
min_count=0
window=0
size=0
model_time = 0.0
infer_time = 0.0
CVtime = []
start = time.time()

if "title" in sys.argv[1]:
    model = Doc2Vec(min_count = 3, window = 5, size = 160, dm = 2, sample = 1e-3, negative = 0, workers = 7)	#title
elif "abstract" in sys.argv[1]:
    model = Doc2Vec(min_count = 4, window = 8, size = 80, dm = 2, sample = 1e-3, negative = 0, workers = 7)	#abstract
else:
    model = Doc2Vec(min_count = 4, window = 4, size = 100, dm = 2, sample = 1e-3, negative = 0, workers = 7)		#function


model.build_vocab(sentences.to_array())

for epoch in range(5000):
    model.train(sentences.sentences_perm())

end = time.time()
model_time = (float(end - start))

X = []
y = []
unique = {}
start = time.time()
#print model.docvecs
for key in model.docvecs.doctags:
    
    x = model.docvecs[key]
    s = '_'.join(str(i) for i in x)
    if not unique.has_key(s):
        unique[s]=1
        X.append(model.docvecs[key])
        l = int(key.split("_")[0])
        y.append(l)
    
end = time.time()
infer_time = (float(end - start))
print len(unique)
print len(y), len(X)

f1LR = []
preLR = []
recalLR = []

f1RF = []
preRF = []
recalRF = []

f1SVM = []
preSVM = []
recalSVM = []

f1GBM = []
preGBM = []
recalGBM = []

for c in range(5):
    print "fold = ", c

    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
        continue
    if np.sum(y_test) == 0:		#only 0 class
        continue

    T = len(y_test)
    P = sum(y_test)
    N = T - P
    start = time.time()

    
    if "title" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 6, gamma = 0.25)	#title
    elif "abstract" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 7, gamma = 0.01)	#abstract
    else:
        cls = SVC(kernel = 'rbf', C = 1, gamma = 0.1)		#function

    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    preSVM.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalSVM.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1SVM.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done SVM"

    end = time.time()
    CVtime.append(float(end - start))

fout=open('Result_DeepLearning_SVM.txt', 'w')


fout.write("\n5 fold SVM:");
fout.write("F1 = %f\n" % np.mean(f1SVM))
fout.write("precision = %f\n" % np.mean(preSVM))
fout.write("recall = %f\n" % np.mean(recalSVM))

time =  model_time +infer_time+ np.mean(CVtime)
fout.write("Model time = "+ str(model_time) +" Sec, Infer time = "+ str(infer_time) +" Sec, Classification time =" + str(np.mean(CVtime))+ "Sec\n")
fout.write("Total time = %f Sec\n" % time)
fout.close();

