# USAGE:
# python Para2Vec_majority_count_protein_level_SVM.py <inputdir_for_deep> <Classlabel_File> <MP File for prediction> <Non_MP FILE for prediction> 
# input dir should contain two files named 0 and 1
# 0 named file contain Non Moonlighting protein text data
# 1 named file contain Moonlighting protein text data
# cutoff between 0 and 1

# gensim modules
from gensim import utils
from gensim.models.doc2vec import TaggedDocument,LabeledSentence
from gensim.models import Doc2Vec

# numpy
import numpy as np

# classifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
# utils
import random
from os import listdir
import sys


def get_prot_label_result(X,y,classifier,prot_test):
   
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
    #probs = classifier.predict_proba(X)
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        text = feature_to_text[x_str]
        #text = bow_to_text[bow_str]
        prots = text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            if not prot_test.has_key(prot):
                continue
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 
    prot_actual = []
    prot_predicted = []
    
    #print len(prot_to_label)
    #print len(prot_predicted_label)
    #print len(prot_test)
    #exit()
    for prot in prot_predicted_label:
        prot_actual.append(prot_to_label[prot])
        MP_count = sum(prot_predicted_label[prot])
#        if MP_count >= len(prot_predicted_label[prot])-MP_count:	#for 50% majority vote	
        if MP_count > len(prot_predicted_label[prot]) * 0.8:
            prot_predicted.append(1)
        else:
            prot_predicted.append(0)            

    T = len(prot_actual)
    P = sum(prot_actual)
    N = T - P
    print metrics.classification_report(prot_actual, prot_predicted)
    precision = ((metrics.precision_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.precision_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    recall = ((metrics.recall_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.recall_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    fscore = ((metrics.f1_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.f1_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    return precision,recall,fscore

class LabeledLineSentence(object):
    def __init__(self, sources):
        self.sources = sources

        flipped = {}

        # make sure that keys are unique
        for key, value in sources.items():
            if value not in flipped:
                flipped[value] = [key]
            else:
                raise Exception('Non-unique prefix encountered')

    def __iter__(self):
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    yield TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no])

    def to_array(self):
        self.sentences = []
        for source, prefix in self.sources.items():
            with utils.smart_open(source) as fin:
                for item_no, line in enumerate(fin):
                    self.sentences.append(TaggedDocument(utils.to_unicode(line).split(), [prefix + '_%s' % item_no]))
        return self.sentences

    def sentences_perm(self):
        random.shuffle(self.sentences)
        return self.sentences

#sources = {'test-neg.txt':'TEST_NEG', 'test-pos.txt':'TEST_POS', 'train-neg.txt':'TRAIN_NEG', 'train-pos.txt':'TRAIN_POS'}
sources = {}
for f in listdir(sys.argv[1]):
    #print f
    if not sources.has_key(f):
        sources[sys.argv[1]+"/"+f]=f
print sources
#exit()
sentences = LabeledLineSentence(sources)

if "title" in sys.argv[1]:
    model = Doc2Vec(min_count = 3, window = 5, size = 160, dm = 2, sample = 1e-3, negative = 0, workers = 7)	#title
elif "abstract" in sys.argv[1]:
    model = Doc2Vec(min_count = 4, window = 8, size = 80, dm = 2, sample = 1e-3, negative = 0, workers = 7)	#abstract
else:
    model = Doc2Vec(min_count = 4, window = 4, size = 100, dm = 2, sample = 1e-3, negative = 0, workers = 7)		#function

#model = Doc2Vec()
model.build_vocab(sentences.to_array())

for epoch in range(1000):
    model.train(sentences.sentences_perm())

print dir(model)
#print model.accuracy
print dir(model.docvecs)


All_prot_label_file = open(sys.argv[2],"r")
prot_to_label = {}

while 1:
    line = All_prot_label_file.readline()
    if not line:
        break
    p,l = line.strip().split()
    prot_to_label[p]=int(l)
    
        
X = []
y = []
MP_file = open(sys.argv[3],"r")
text_to_prot = {}
text_to_label = {}
feature_to_text = {}
texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        l = utils.to_unicode(text).split()
        x =  model.infer_vector(l)
        X.append(x)
        y.append(1)
        x_str = " ".join([str(j) for j in x])
        feature_to_text[x_str]=text
        #texts.append(text.split())
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open(sys.argv[4],"r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        l = utils.to_unicode(text).split()
        x =  model.infer_vector(l)
        X.append(x)
        y.append(0)
        x_str = " ".join([str(j) for j in x])
        feature_to_text[x_str]=text
        #texts.append(text.split())
    else:
        text_to_prot[text].add(id)



f1LR = []
preLR = []
recalLR = []

f1RF = []
preRF = []
recalRF = []

f1SVM = []
preSVM = []
recalSVM = []

f1GBM = []
preGBM = []
recalGBM = []

for c in range(5):
    print "fold = ", c
    prot_train = {}
    prot_test = {}
    prot_index = np.asarray(range(0,len(prot_to_label),1))
    np.random.seed(0+c*10)
    np.random.shuffle(prot_index)
    
    test_fold = 20*len(prot_to_label)/100
    for i in range(0,len(prot_to_label)):
        prot = prot_to_label.keys()[prot_index[i]]
        if i < test_fold:
            prot_test[prot]=prot_to_label[prot]
        else:
            prot_train[prot]=prot_to_label[prot]

    X_train = []
    X_test = []
    y_train = []
    y_test = []
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        text = feature_to_text[x_str]
        prots = text_to_prot[text]
        if len(prots.intersection(set(prot_test.keys()))) > 0:
            X_test.append(x)
            y_test.append(y[i])
        else:
            X_train.append(x)
            y_train.append(y[i])
    #X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

    
    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
        continue
    if np.sum(y_test) == 0:		#only 0 class
        continue

    print len(X_test)
    print len(X_train)
    #exit()

    
    if "title" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 6, gamma = 0.25)	#title
    elif "abstract" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 7, gamma = 0.01)	#abstract
    else:
        cls = SVC(kernel = 'rbf', C = 1, gamma = 0.1)		#function

    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    precision,recall,fscore = get_prot_label_result(X,y,cls,prot_test)    
    preSVM.append(precision)
    recalSVM.append(recall)
    f1SVM.append(fscore)
    print "done SVM"

fout = open('Result_Deeplearning_SVM.txt', 'a')


fout.write("\n5 fold SVM: " +" "+sys.argv[1]+" " + sys.argv[5]+"\n");
fout.write("F1 = %f\n" % np.mean(f1SVM))
fout.write("precision = %f\n" % np.mean(preSVM))
fout.write("recall = %f\n" % np.mean(recalSVM))

fout.close();


