# USAGE:
# python TFIDF_protein_level_majority_vote.py <Classlabel_File> <MP File> <Non_MP FILE>

#example:

# python TFIDF_protein_level_majority_vote.py class_labels/All_prot_cls_lbl_Title ../../../Data/Cleaned/Title/title_MP.dat.processed ../../../Data/Cleaned/Title/title_NMP.dat.processed
# Similar for abstract 

import sys
from gensim import corpora, models, similarities
from pprint import pprint
from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
import numpy as np


output = open("MPprediction_TFIDF","w")

def get_label_text(bow_str):

    text = bow_to_text[bow_str]
    label = text_to_label[text]
    return label

def get_prot_label_result(X,y,classifier,prot_test):
   
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
   
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = feature_to_bow[x_str]
        text = bow_to_text[bow_str]
        prots = text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            if not prot_test.has_key(prot):
                continue
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 
    prot_actual = []
    prot_predicted = []
    
    
    for prot in prot_predicted_label:
        prot_actual.append(prot_to_label[prot])
        MP_count = sum(prot_predicted_label[prot])
        
        if MP_count >= len(prot_predicted_label[prot]) *0.5: #change 0.5 for other cutoffs. see paper for details
            prot_predicted.append(1)
            if prot_to_label[prot]==1:
                output.write(prot+" 1"+"\n")
        else:
            prot_predicted.append(0)
            if prot_to_label[prot]==1:
                output.write(prot+" 0"+"\n")            
   
    T = len(prot_actual)
    P = sum(prot_actual)
    N = T - P
    print metrics.classification_report(prot_actual, prot_predicted)
    precision = ((metrics.precision_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.precision_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    recall = ((metrics.recall_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.recall_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    fscore = ((metrics.f1_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.f1_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    return precision,recall,fscore

All_prot_label_file = open(sys.argv[1],"r")
prot_to_label = {}

while 1:
    line = All_prot_label_file.readline()
    if not line:
        break
    p,l = line.strip().split()
    prot_to_label[p]=int(l)
    
         

MP_file = open(sys.argv[2],"r")
text_to_prot = {}
text_to_label = {}
texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open(sys.argv[3],"r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

dictionary = corpora.Dictionary(texts)
bow_to_text = {}

corpus = [] 
for text in texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not bow_to_text.has_key(store_text):
        bow_to_text[str(bow)] = store_text
    corpus.append(bow)


tfidf = models.TfidfModel(corpus)

X = []
y = []
feature_to_bow = {}
for bow in corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    hash_ = dict(doc)
    label = get_label_text(bow_str)
   
    y.append(label)
    x = []
    for word,id in dictionary.token2id.iteritems():
        if id in hash_:
            x.append(hash_[id])
        else:
            x.append(0)
    x_str = " ".join([str(i) for i in x])
    feature_to_bow[x_str]=bow_str
    X.append(x)

f1LR = []
preLR = []
recalLR = []

f1RF = []
preRF = []
recalRF = []

f1SVM = []
preSVM = []
recalSVM = []

f1GBM = []
preGBM = []
recalGBM = []

for c in range(3):
    #print "fold = ", c
    prot_train = {}
    prot_test = {}
    prot_index = np.asarray(range(0,len(prot_to_label),1))
    np.random.seed(0+c*10)
    np.random.shuffle(prot_index)
    
    test_fold = 20*len(prot_to_label)/100
    for i in range(0,len(prot_to_label)):
        prot = prot_to_label.keys()[prot_index[i]]
        if i < test_fold:
            prot_test[prot]=prot_to_label[prot]
            #print prot,prot_to_label[prot]
        else:
            prot_train[prot]=prot_to_label[prot]
            #print prot, prot_to_label[prot]
    
    X_train = []
    X_test = []
    y_train = []
    y_test = []
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = feature_to_bow[x_str]
        text = bow_to_text[bow_str]
        prots = text_to_prot[text]
        if len(prots.intersection(set(prot_test.keys()))) > 0:
            X_test.append(x)
            y_test.append(y[i])
        else:
            X_train.append(x)
            y_train.append(y[i])
    #X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

    
    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
        continue
    if np.sum(y_test) == 0:		#only 0 class
        continue

    
    classifier = LogisticRegression(C = 1, solver = 'liblinear', max_iter = 100)
    classifier.fit(X_train, y_train)
    predicted = classifier.predict(X_test)
    
   
    precision,recall,fscore = get_prot_label_result(X_test,y_test,classifier,prot_test)    
    preLR.append(precision)
    recalLR.append(recall)
    f1LR.append(fscore)

    print "done LR"
   
    cls = RandomForestClassifier(n_estimators = 500)
    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    precision,recall,fscore = get_prot_label_result(X_test,y_test,cls,prot_test)    
    preRF.append(precision)
    recalRF.append(recall)
    f1RF.append(fscore)
    print "done RF"
    
    # corss validated classifier
    if "title" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 6, gamma = 0.5)	#title
    else:
        cls = SVC(kernel = 'linear', C = 2)			#abstract/function
       
   
    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    precision,recall,fscore = get_prot_label_result(X_test,y_test,cls,prot_test)    
    preSVM.append(precision)
    recalSVM.append(recall)
    f1SVM.append(fscore)
    print "done SVM"

    cls = GradientBoostingClassifier(learning_rate=0.3, n_estimators = 100)
    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    precision,recall,fscore = get_prot_label_result(X_test,y_test,cls,prot_test) 
    preGBM.append(precision)
    recalGBM.append(recall)
    f1GBM.append(fscore)
    print "done GBM"

fout = open('Result_TFIDF.txt', 'a')

fout.write("5 fold LogisticRegression:");
fout.write("F1 = %f\n" % np.mean(f1LR))
fout.write("precision = %f\n" % np.mean(preLR))
fout.write("recall = %f\n" % np.mean(recalLR))

fout.write("\n5 fold Random Forest:");
fout.write("F1 = %f\n" % np.mean(f1RF))
fout.write("precision = %f\n" % np.mean(preRF))
fout.write("recall = %f\n" % np.mean(recalRF))

fout.write("\n5 fold SVM:");
fout.write("F1 = %f\n" % np.mean(f1SVM))
fout.write("precision = %f\n" % np.mean(preSVM))
fout.write("recall = %f\n" % np.mean(recalSVM))

fout.write("\n5 fold GBM:");
fout.write("F1 = %f\n" % np.mean(f1GBM))
fout.write("precision = %f\n" % np.mean(preGBM))
fout.write("recall = %f\n" % np.mean(recalGBM))

fout.close();
print np.mean(f1LR), np.mean(f1RF), np.mean(f1SVM), np.mean(f1GBM)

